/* LCD Big Number Clock Demo
 * http://frank.circleofcurrent.com/musicclock/
 * Copyright (c) 2011 Frank Zhao
 *
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// please see http://www.nongnu.org/avr-libc/user-manual/modules.html for info on these modules

#include <avr/io.h> // allows access to AVR built-in peripherals and registers
#include <avr/pgmspace.h> // allows access to flash memory
#include <util/delay.h> // provides delays
#include <stdio.h> // allows formatted debug output
#include <avr/power.h> // used for setting clock prescaler
#include "libs/usb_serial.h" // provides USB-to-serial functionality
#include "lcd.h" // provides LCD display functionality

// this is a wrapper for usb_putchar that allows me to use printf from stdio.h
static inline int debug_putChar(char c, FILE* stream)
{
    return usb_serial_putchar(c); // refer to http://www.pjrc.com/teensy/usb_serial.html
}

// this is a wrapper for lcd_sendchar that allows me to use printf from stdio.h
static inline int lcd_putChar(char c, FILE* stream)
{
    lcd_sendChar(c);
}

// setup debug stream for printf from stdio.h
static FILE debug_stdout = FDEV_SETUP_STREAM(debug_putChar, NULL, _FDEV_SETUP_WRITE);

// setup stream for printf from stdio.h using the LCD display
static FILE lcd_stdout = FDEV_SETUP_STREAM(lcd_putChar, NULL, _FDEV_SETUP_WRITE);

int main()
{
    clock_prescale_set(clock_div_2); // this is because we are running at 3.3V
    // see http://www.nongnu.org/avr-libc/user-manual/group__avr__power.html and http://www.pjrc.com/teensy/3volt.html
    
    usb_init(); // initialize usb obviously
    
    while (usb_configured() == 0) _delay_ms(5); // waits until USB is ready
    
    // 5 seconds for you to connect hyperterminal
    // while flashing backlight
    LCD_DDR |= _BV(LCD_PIN_LED);
    for (int i = 0; i < 5; i++)
    {
        lcd_backlightOn();
        _delay_ms(500);
        lcd_backlightOff();
        _delay_ms(500);
    }
    
    // here's the start of the demonstration
    fprintf(&debug_stdout, "Initializing LCD\r\n");
    lcd_init();
    lcd_backlightOn();
    
    while (1)
    {
        lcd_printTime(01, 23, 0);
        _delay_ms(5000);
        lcd_printTime(23, 45, 0);
        _delay_ms(5000);
        lcd_printTime(23, 67, 0);
        _delay_ms(5000);
        lcd_printTime(23, 89, 0);
        _delay_ms(5000);
    }
    
    return 0;
}